package ru.ulstu.is.sbapp.socialNetwork.controller;


import org.springframework.web.bind.annotation.*;
import ru.ulstu.is.sbapp.configuration.WebConfiguration;
import ru.ulstu.is.sbapp.socialNetwork.dto.MusicDTO;
import ru.ulstu.is.sbapp.socialNetwork.services.MusicService;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/musics")
public class MusicController {

    private final MusicService musicService;

    public MusicController(MusicService musicService){
        this.musicService = musicService;
    }

    @GetMapping("/{id}")
    public MusicDTO getMusic(@PathVariable Long id){
        return new MusicDTO(musicService.findMusic(id));
    }

    @GetMapping("")
    public List<MusicDTO> getMusics() {
        return musicService.findAllMusics().stream().map(MusicDTO::new).toList();
    }

    @PostMapping("")
    public MusicDTO createMusic(@RequestBody @Valid MusicDTO music) {
        return new MusicDTO(musicService.addMusic(music.getName()));
    }

    @PatchMapping("/{id}")
    public MusicDTO updateMusic(@PathVariable Long id,
                                @RequestBody @Valid MusicDTO music) {
        return new MusicDTO(musicService.updateMusic(id, music.getName()));
    }

    @DeleteMapping("/{id}")
    public MusicDTO deleteMusic(@PathVariable Long id) {
        return new MusicDTO(musicService.deleteMusic(id));
    }

}
