package ru.ulstu.is.sbapp.socialNetwork.services;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ulstu.is.sbapp.socialNetwork.models.Community;
import ru.ulstu.is.sbapp.socialNetwork.repository.CommunityRepository;

@Service
public class CommunityService {

    private CommunityRepository repo;

    public CommunityService(CommunityRepository repo){
        this.repo = repo;
    }

    @Transactional
    public Community addCommunity(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Community name is null or empty");
        }
        Community community = new Community(name);
        return repo.save(community);
    }

    @Transactional(readOnly = true)
    public Community findCommunity(Long id) {
        final Optional<Community> сommunity = repo.findById(id);
        if (сommunity.isEmpty()) {
            throw new EntityNotFoundException(String.format("Community with id [%s] is not found", id));
        }
        return сommunity.get();
    }

    @Transactional(readOnly = true)
    public List<Community> findAllCommunities() {
        return repo.findAll();
    }


    @Transactional
    public Community updateCommunity(Long id, String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Music name is null or empty");
        }
        final Optional<Community> currentCommunityOpt = repo.findById(id);

        if(currentCommunityOpt.isEmpty()) {
            return null;
        }

        final Community currentCommunity = currentCommunityOpt.get();

        currentCommunity.setName(name);
        return repo.save(currentCommunity);
    }
    @Transactional
    public Community findByName(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Genre name is null or empty");
        }
        return repo.findByName(name);
    }

    @Transactional
    public Community deleteCommunity(Long id) {
        final Optional<Community> currentCommunity = repo.findById(id);
        if(currentCommunity.isEmpty()) {
            return null;
        }
        repo.deleteById(id);
        return currentCommunity.get();
    }

    @Transactional
    public void deleteAllCommunities() {
        repo.deleteAll();
    }

}

