package ru.ulstu.is.sbapp.socialNetwork.services;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import ru.ulstu.is.sbapp.socialNetwork.models.Community;
import ru.ulstu.is.sbapp.socialNetwork.models.Music;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.repository.CommunityRepository;
import ru.ulstu.is.sbapp.socialNetwork.repository.MusicRepository;

import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.Optional;


@Service
public class MusicService {

    private MusicRepository repo;

    public MusicService(MusicRepository repo) {
        this.repo = repo;
    }
    @Transactional
    public Music addMusic(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Music name is null or empty");
        }

        Music music = new Music(name);

        return repo.save(music);
    }

    @Transactional(readOnly = true)
    public Music findMusic(Long id) {

        final Optional<Music> music = repo.findById(id);
        if (music.isEmpty()) {
            throw new EntityNotFoundException(String.format("Music with id [%s] is not found", id));
        }
        return music.get();
    }

    @Transactional(readOnly = true)
    public List<Music> findAllMusics() {
        return repo.findAll();
    }

    @Transactional
    public Music updateMusic(Long id, String name) {

        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Music name is null or empty");
        }
        final Optional<Music> currentMusicOpt = repo.findById(id);

        if(currentMusicOpt.isEmpty()) {
            return null;
        }

        final Music currentMusic = currentMusicOpt.get();

        currentMusic.setName(name);
        return repo.save(currentMusic);
    }
    @Transactional
    public Music findByName(String name) {
        if (!StringUtils.hasText(name)) {
            throw new IllegalArgumentException("Music name is null or empty");
        }
        return repo.findByName(name);
    }

    @Transactional
    public Music deleteMusic(Long id) {

        final Optional<Music> currentMusic = repo.findById(id);
        if(currentMusic.isEmpty()) {
            return null;
        }

        repo.deleteById(id);
        return currentMusic.get();
    }

    @Transactional
    public void deleteAllMusics() {
        repo.deleteAll();
    }

}

