package ru.ulstu.is.sbapp;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ulstu.is.sbapp.socialNetwork.models.Music;
import ru.ulstu.is.sbapp.socialNetwork.services.MusicService;
import ru.ulstu.is.sbapp.socialNetwork.services.MyUserService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaMusicTest {
    private static final Logger log = LoggerFactory.getLogger(JpaUserTest.class);
    @Autowired
    MyUserService userService;

    @Autowired
    MusicService musicService;

    @Test
    void testMusicCreate() {
        musicService.deleteAllMusics();
        final Music music = musicService.addMusic("Song");
        log.info(music.toString());
        Assertions.assertNotNull(music.getId());
    }

    @Test
    void testMusicReadNotFound() {
        musicService.deleteAllMusics();
        Assertions.assertThrows(EntityNotFoundException.class, () -> musicService.findMusic(-1L));
    }

    @Test
    void findMusic() {
        musicService.deleteAllMusics();
        final Music music = musicService.addMusic("Song");
        log.info(musicService.toString());
        final Music findMusic = musicService.findMusic(music.getId());
        log.info(findMusic.toString());
        Assertions.assertEquals(music.toString(), findMusic.toString());
    }

    @Test
    void testReadAll() {
        musicService.deleteAllMusics();
        final Music music = musicService.addMusic("Song");
        log.info(music.toString());
        final List<Music> musics = musicService.findAllMusics();
        Assertions.assertEquals(musics.size(), 1);
    }

    @Test
    void updateMusic() {
        musicService.deleteAllMusics();
        final Music music = musicService.addMusic("Song");
        musicService.updateMusic(music.getId(), "Song 2");
        final Music findMusic = musicService.findMusic(music.getId());
        log.info(findMusic.toString());
        Assertions.assertEquals(findMusic.getName(), "Song 2");
    }

    @Test
    void deleteMusic() {
        musicService.deleteAllMusics();
        final Music music = musicService.addMusic("Song");
        log.info(music.toString());
        musicService.deleteMusic(music.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> userService.findUser(music.getId()));
    }
}

