package ru.ulstu.is.sbapp;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import ru.ulstu.is.sbapp.socialNetwork.models.Community;
import ru.ulstu.is.sbapp.socialNetwork.models.Music;
import ru.ulstu.is.sbapp.socialNetwork.models.UserModel;
import ru.ulstu.is.sbapp.socialNetwork.services.CommunityService;
import ru.ulstu.is.sbapp.socialNetwork.services.MusicService;
import ru.ulstu.is.sbapp.socialNetwork.services.MyUserService;

import javax.persistence.EntityNotFoundException;
import java.util.List;

@SpringBootTest
public class JpaUserTest {
    private static final Logger log = LoggerFactory.getLogger(JpaUserTest.class);
    @Autowired
    MyUserService userService;

    @Autowired
    MusicService musicService;

    @Autowired
    CommunityService communityService;

    @Test
    void testUserCreate() {
        userService.deleteAllUsers();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        log.info(user.toString());
        Assertions.assertNotNull(user.getId());
    }

    @Test
    void testUserReadNotFound() {
        userService.deleteAllUsers();
        Assertions.assertThrows(EntityNotFoundException.class, () -> userService.findUser(-1L));
    }

    @Test
    void findUser() {
        userService.deleteAllUsers();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        log.info(user.toString());
        final UserModel findUser = userService.findUser(user.getId());
        log.info(findUser.toString());
        Assertions.assertEquals(user.toString(), findUser.toString());
    }

    @Test
    void updateUser() {
        userService.deleteAllUsers();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        userService.updateUser(user.getId(), "its_mailov 2");
        final UserModel findUser = userService.findUser(user.getId());
        log.info(findUser.toString());
        Assertions.assertEquals(findUser.getName(), "its_mailov 2");

    }

    @Test
    void testReadAll() {
        userService.deleteAllUsers();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        log.info(user.toString());
        final List<UserModel> users = userService.findAllUsers();
        Assertions.assertEquals(users.size(), 1);
    }

    @Test
    void deleteUser() {
        userService.deleteAllUsers();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        log.info(user.toString());
        userService.deleteUser(user.getId());
        Assertions.assertThrows(EntityNotFoundException.class, () -> userService.findUser(user.getId()));
    }




    @Test
    void addMusic() {
        userService.deleteAllUsers();
        musicService.deleteAllMusics();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        final Music music = musicService.addMusic("Song");

        userService.addMusic(user.getId(), music.getId());

        final UserModel findUser = userService.findUser(user.getId());
        Assertions.assertEquals(findUser.getMusics().get(0).toString(), music.toString());
    }

    @Test
    void addGroup() {
        userService.deleteAllUsers();
        musicService.deleteAllMusics();
        final UserModel user = userService.addUser("its_mailov", "Moscow");
        final Community community = communityService.addCommunity("GPT");

        userService.addCommunity(user.getId(), community.getId());

        final UserModel findUser = userService.findUser(user.getId());
        Assertions.assertEquals(findUser.getGroups().get(0).getId(), community.getId());

    }
}
